@@ -352,15 +352,16 @@ namespace aptitude
             components.push_back(transcode(mode));
 
           if(total_size != 0 && !complete)
-            components.push_back((wformat(L"%lu/%sB %lu%%")
-                                  % current_size
+            components.push_back((wformat(L"%sB/%sB %lu%%")
+                                  % transcode(SizeToStr(current_size))
                                   % transcode(SizeToStr(total_size))
                                   % as_percent( ((double) current_size) / total_size)).str());
           else if(current_size != 0)
             // The old download indicator displayed a size of 0 if
             // current_size was 0.  I figure if we have no total size
             // and we haven't downloaded anything, there's no poing.
-            components.push_back((wformat(L"%lu") % current_size).str());
+            components.push_back((wformat(L"%sB")
+                                  % transcode(SizeToStr(current_size))).str());
 
           output += join(components, L" ");
         }
