@@ -5,6 +5,8 @@
 #ifndef CMDLINE_SHOW_BROKEN_H
 #define CMDLINE_SHOW_BROKEN_H
 
+#include <apt-pkg/pkgcache.h>
+
 /** \file cmdline_show_broken.h
  */
 
@@ -16,4 +18,7 @@
  */
 bool show_broken();
 
+/** Shows broken dependencies for a single package */
+void show_broken_deps(pkgCache::PkgIterator pkg);
+
 #endif // CMDLINE_SHOW_BROKEN
