@@ -27,6 +27,7 @@
 #include <apt-pkg/cachefile.h>
 #include <apt-pkg/clean.h>
 #include <apt-pkg/error.h>
+#include <apt-pkg/algorithms.h>
 
 #include <cwidget/generic/util/exception.h>
 #include <cwidget/generic/util/ssprintf.h>
@@ -78,6 +79,8 @@ bool download_update_manager::prepare(OpProgress &progress,
       return false;
     }
 
+  stat = &acqlog;
+
   // Abort here so we don't spew random messages below.
   if(_error->PendingError())
     return false;
@@ -94,17 +97,23 @@ bool download_update_manager::prepare(OpProgress &progress,
 	}
     }
 
-  fetcher = new pkgAcquire();
-  fetcher->Setup(&acqlog);
+  return true;
+}
 
-  if(!src_list.GetIndexes(fetcher))
-    {
-      delete fetcher;
-      fetcher = NULL;
-      return false;
-    }
-  else
-    return true;
+pkgAcquire::RunResult download_update_manager::do_download()
+{
+  if(ListUpdate(*stat, src_list) == false)
+    return pkgAcquire::Failed;
+
+  return pkgAcquire::Continue;
+}
+
+pkgAcquire::RunResult download_update_manager::do_download(int PulseInterval)
+{
+  if(ListUpdate(*stat, src_list, PulseInterval) == false)
+    return pkgAcquire::Failed;
+
+  return pkgAcquire::Continue;
 }
 
 // TODO: this should be lifted to generic code.
@@ -283,46 +292,6 @@ void download_update_manager::finish(pkgAcquire::RunResult res,
       return;
     }
 
-  bool transientNetworkFailure = false;
-  result rval = success;
-
-  // We need to claim that the download failed if any source failed,
-  // and invoke Finished() on any failed items.  Also, we shouldn't
-  // clean the package lists if any individual item failed because it
-  // makes users grumpy (see Debian bugs #201842 and #479620).
-  //
-  // See also apt-get.cc.
-  for(pkgAcquire::ItemIterator it = fetcher->ItemsBegin();
-      it != fetcher->ItemsEnd(); ++it)
-    {
-      if((*it)->Status == pkgAcquire::Item::StatDone)
-	continue;
-
-      (*it)->Finished();
-
-      if((*it)->Status == pkgAcquire::Item::StatTransientNetworkError)
-	{
-	  transientNetworkFailure = true;
-	  continue;
-	}
-
-      // Q: should I display an error message for this source?
-      rval = failure;
-    }
-
-  // Clean old stuff out
-  std::string listsdir = aptcfg->FindDir("Dir::State::lists");
-  if(rval == success && !transientNetworkFailure &&
-     aptcfg->FindB("APT::Get::List-Cleanup", true) == true &&
-     aptcfg->FindB("APT::List-Cleanup", true) == true &&
-     (fetcher->Clean(listsdir) == false ||
-      fetcher->Clean(listsdir + "partial/") == false))
-    {
-      _error->Error(_("Couldn't clean out list directories"));
-      k(failure);
-      return;
-    }
-
   // Rebuild the apt caches as done in apt-get.  cachefile is scoped
   // so it dies before we possibly-reload the cache.  This will do a
   // little redundant work in visual mode, but avoids lots of
@@ -331,7 +300,6 @@ void download_update_manager::finish(pkgAcquire::RunResult res,
     pkgCacheFile cachefile;
     if(!cachefile.BuildCaches(progress, true))
       {
-	_error->Error(_("Couldn't rebuild package cache"));
 	k(failure);
 	return;
       }
@@ -369,7 +337,7 @@ void download_update_manager::finish(pkgAcquire::RunResult res,
     }
   else
     {
-      progress->OverallProgress(0, 0, 1, _("Updating debtags database..."));
+      progress->OverallProgress(0, 0, 1, _("Updating debtags database"));
 
       std::string debtags_options = aptcfg->Find(PACKAGE "::Debtags-Update-Options", "--local");
 
@@ -440,7 +408,7 @@ void download_update_manager::finish(pkgAcquire::RunResult res,
       post_autoclean_hook();
     }
 
-  k(rval);
+  k(success);
   return;
 }
 
