@@ -1,6 +1,6 @@
 // download_update_manager.h                   -*-c++-*-
 //
-//   Copyright (C) 2005 Daniel Burrows
+//   Copyright (C) 2005, 2008, 2010 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -38,6 +38,7 @@ class download_update_manager : public download_manager
   download_signal_log *log;
 
   pkgSourceList src_list;
+  pkgAcquireStatus *stat;
 
 public:
   /** Create a new manager.  Note that acqlog and signallog may or may
@@ -71,8 +72,12 @@ public:
 	       pkgAcquireStatus &acqlog,
 	       download_signal_log *signallog);
 
-  result finish(pkgAcquire::RunResult result,
-		OpProgress &progress);
+  pkgAcquire::RunResult do_download();
+  pkgAcquire::RunResult do_download(int PulseInterval);
+
+  void finish(pkgAcquire::RunResult res,
+	      OpProgress *progress,
+	      const sigc::slot1<void, result> &k);
 
   /** A signal that is invoked after an automatic 'forget new'
    *  operation.
