@@ -255,7 +255,7 @@ namespace aptitude
 	case atomic:
 	  // TODO: do something special based on the type, like we
 	  // used to before the recent changes.
-	  return 0;
+	  return match_string.empty() ? 0 : 1;
 
 	case regexp:
 	  return regexp_matches.size();
@@ -281,6 +281,10 @@ namespace aptitude
       switch(tp)
 	{
 	case atomic:
+	  if(group_num >= (match_string.empty() ? 0 : 1))
+	    throw MatchingException("Can't retrieve match information: the group number is out of bounds.");
+	  return std::string(match_string);
+
 	case dependency:
 	case provides:
 	  throw MatchingException("Can't retrieve match information: the group number is out of bounds.");
@@ -871,12 +875,15 @@ namespace aptitude
 		  // The rest correspond directly to find_pkg_state() return values.
 		case pattern::action_reinstall:
 		  matches = find_pkg_state(pkg, cache) == pkg_reinstall;
+		  break;
 
 		case pattern::action_upgrade:
 		  matches = find_pkg_state(pkg, cache) == pkg_upgrade;
+		  break;
 
 		case pattern::action_downgrade:
 		  matches = find_pkg_state(pkg, cache) == pkg_downgrade;
+		  break;
 
 		case pattern::action_keep:
 		  matches = cache[pkg].Keep();
@@ -894,6 +901,20 @@ namespace aptitude
 
 	    break;
 
+	  case pattern::architecture:
+	    if(!target.get_has_version())
+	      return NULL;
+
+	    {
+	      pkgCache::VerIterator ver(target.get_version_iterator(cache));
+              if(p->get_architecture_architecture().empty() == true ||
+		 p->get_architecture_architecture() == ver.Arch())
+                return match::make_atomic(p, ver.Arch());
+              else
+                return NULL;
+	    }
+	    break;
+
 	  case pattern::automatic:
 	    {
 	      pkgCache::PkgIterator pkg(target.get_package_iterator(cache));
@@ -1142,6 +1163,18 @@ namespace aptitude
 	    return NULL;
 	    break;
 
+	  case pattern::foreign_architecture:
+	    if(!target.get_has_version())
+	      return NULL;
+
+	    {
+	      pkgCache::VerIterator ver(target.get_version_iterator(cache));
+	      if(aptitude::apt::is_foreign_arch(ver))
+		return match::make_atomic(p, ver.Arch());
+	      else
+		return NULL;
+	    }
+
 	  case pattern::garbage:
 	    if(!target.get_has_version())
 	      return NULL;
@@ -1182,6 +1215,38 @@ namespace aptitude
 	      }
 	    break;
 
+	  case pattern::multiarch:
+	    if(!target.get_has_version())
+	      return NULL;
+	    else
+	      {
+		bool matches = false;
+                const pattern::multiarch_type type = p->get_multiarch_multiarch_type();
+
+                switch(target.get_ver()->MultiArch)
+                  {
+                  case pkgCache::Version::Foreign:
+                  case pkgCache::Version::AllForeign:
+                    matches = (type == pattern::multiarch_foreign);
+                    break;
+                  case pkgCache::Version::Same:
+                    matches = (type == pattern::multiarch_same);
+                    break;
+                  case pkgCache::Version::Allowed:
+                  case pkgCache::Version::AllAllowed:
+                    matches = (type == pattern::multiarch_allowed);
+                    break;
+                  default:
+                    matches = (type == pattern::multiarch_none);
+                  }
+
+		if(matches)
+		  return match::make_atomic(p);
+		else
+		  return NULL;
+	      }
+	    break;
+
 	  case pattern::name:
 	    return evaluate_regexp(p,
 				   p->get_name_regex_info(),
@@ -1189,6 +1254,18 @@ namespace aptitude
 				   debug);
 	    break;
 
+	  case pattern::native_architecture:
+	    if(!target.get_has_version())
+	      return NULL;
+
+	    {
+	      pkgCache::VerIterator ver(target.get_version_iterator(cache));
+	      if(aptitude::apt::is_native_arch(ver))
+		return match::make_atomic(p, ver.Arch());
+	      else
+		return NULL;
+	    }
+
 	  case pattern::new_tp:
 	    if(!target.get_has_version())
 	      return NULL;
@@ -2013,6 +2090,7 @@ namespace aptitude
 
 	  case pattern::archive:
 	  case pattern::action:
+	  case pattern::architecture:
 	  case pattern::automatic:
 	  case pattern::bind:
 	  case pattern::broken:
@@ -2026,11 +2104,14 @@ namespace aptitude
 	  case pattern::equal:
 	  case pattern::exact_name:
 	  case pattern::false_tp:
+	  case pattern::foreign_architecture:
 	  case pattern::garbage:
 	  case pattern::install_version:
 	  case pattern::installed:
 	  case pattern::maintainer:
+	  case pattern::multiarch:
 	  case pattern::name:
+	  case pattern::native_architecture:
 	  case pattern::new_tp:
 	  case pattern::obsolete:
 	  case pattern::origin:
@@ -2223,6 +2304,7 @@ namespace aptitude
 
 	  case pattern::archive:
 	  case pattern::action:
+	  case pattern::architecture:
 	  case pattern::automatic:
 	  case pattern::bind:
 	  case pattern::broken:
@@ -2235,11 +2317,14 @@ namespace aptitude
 	  case pattern::essential:
 	  case pattern::equal:
 	  case pattern::false_tp:
+	  case pattern::foreign_architecture:
 	  case pattern::garbage:
 	  case pattern::install_version:
 	  case pattern::installed:
 	  case pattern::maintainer:
+	  case pattern::multiarch:
 	  case pattern::name:
+	  case pattern::native_architecture:
 	  case pattern::new_tp:
 	  case pattern::obsolete:
 	  case pattern::origin:
@@ -2345,6 +2430,7 @@ namespace aptitude
 
 	  case pattern::archive:
 	  case pattern::action:
+	  case pattern::architecture:
 	  case pattern::automatic:
 	  case pattern::bind:
 	  case pattern::broken:
@@ -2357,11 +2443,14 @@ namespace aptitude
 	  case pattern::essential:
 	  case pattern::equal:
 	  case pattern::false_tp:
+	  case pattern::foreign_architecture:
 	  case pattern::garbage:
 	  case pattern::install_version:
 	  case pattern::installed:
 	  case pattern::maintainer:
+	  case pattern::multiarch:
 	  case pattern::name:
+	  case pattern::native_architecture:
 	  case pattern::new_tp:
 	  case pattern::obsolete:
 	  case pattern::origin:
@@ -2540,6 +2629,7 @@ namespace aptitude
 	    // Various non-Xapian patterns, along with ?term.
 	  case pattern::archive:
 	  case pattern::action:
+	  case pattern::architecture:
 	  case pattern::automatic:
 	  case pattern::bind:
 	  case pattern::broken:
@@ -2553,11 +2643,14 @@ namespace aptitude
 	  case pattern::equal:
 	  case pattern::exact_name:
 	  case pattern::false_tp:
+	  case pattern::foreign_architecture:
 	  case pattern::garbage:
 	  case pattern::install_version:
 	  case pattern::installed:
 	  case pattern::maintainer:
+	  case pattern::multiarch:
 	  case pattern::name:
+	  case pattern::native_architecture:
 	  case pattern::new_tp:
 	  case pattern::obsolete:
 	  case pattern::origin:
@@ -2828,6 +2921,7 @@ namespace aptitude
 
 	  case pattern::archive:
 	  case pattern::action:
+	  case pattern::architecture:
 	  case pattern::automatic:
 	  case pattern::bind:
 	  case pattern::broken:
@@ -2840,11 +2934,14 @@ namespace aptitude
 	  case pattern::essential:
 	  case pattern::equal:
 	  case pattern::false_tp:
+	  case pattern::foreign_architecture:
 	  case pattern::garbage:
 	  case pattern::install_version:
 	  case pattern::installed:
 	  case pattern::maintainer:
+	  case pattern::multiarch:
 	  case pattern::name:
+	  case pattern::native_architecture:
 	  case pattern::new_tp:
 	  case pattern::obsolete:
 	  case pattern::origin:
